///@file documentation.h Doxygen documentation bits

/// @defgroup gDebug Useful debugging functions.

/// @defgroup gUtility General utility functions.

/// @defgroup gHMM Generic hidden Markov model solver.
///
/// The notation follows `A tutorial on hidden Markov models and selected applications in speech recognition', Rabiner, 1989
///

/// @defgroup gStorm Storm classes

// @defgroup gMultiSpotDrift Storm classes specific to multispot processing with drift

/// @defgroup gStormImages Storm imagery classes (basic image processing)

// @defgroup gGraphics Graphical display

// @defgroup gUserInterface General user interface classes

/// @file multispot5.cc Fit spots to the data

/// @file multispot5_headless.cc FitSpots driver for entierly headless (batch) operation

/// @file multispot5_gui.cc FitSpots driver for interactive (GUI) operation and debugging

///@file mt19937.h Mersenne twister interface code

///@file randomc.h 

///@file mersenne.cpp Agner Fogg's Mersenne Twister implementation.

/// @file debug.h  Debugging bits

/// @file debug.cc Debugging bits

/// @file utility.h Utility bits.

/// @file utility.cc Utility bits.

/// @file storm_imagery.cc Code dealing with storm imagery (low level).

/// @file storm_imagery.h Code dealing with storm imagery (low level).

/// @file storm.h Code dealing with storm imagery (high level).

/// @file forward_algorithm.h Contains an implementation fo the forward algorithm.


/// @defgroup gMultiSpot Storm classes specific to multispot processing
// 
// Having completely smooth drift is too memory intensive.
// 
// The drift model is linear, approximated as piecewise constant.  Generally there
// are <i>F</i> frames and these are divided up into <i>S</i> steps. The equation
// for finding the step \e s from the frame \e i is:
// \f[
// i	s = \lfloor \frac{f S}{F} \rfloor.
// \f]
// The inverse is defined to be:
// \f[
// 	f \approx \frac{s + \frac{1}{2}}{S}F
// \f]
//


/** \mainpage 3B Microscopy Analysis

\section sIntro Introduction

This project contains the reference implementation of the 3B microscopy analysis method.
Please refer to <a href="http://www.coxphysics.com/3b">the project website</a> for more information
on the method.

The program (multispot5_headless) performs the
analysis. This program needs to be run from the commandline.

\section Compiling

In order to compile the project, you will need to download and install the
following libraries:
<ul>
<li> TooN http://www.edwardrosten.com/cvd/toon.html
<li> libcvd http://www.edwardrosten.com/cvd/
<li> gvars3 http://www.edwardrosten.com/cvd/gvars3.html
</ul>
The program is portable and is well tested under Linux and OSX. It will also
compile under Windows using cygwin or MinGW.

\section sExample Example of usage

We have provided a set of test data on the website. Download and unpack the zip
file. It will create a new directory called test data with the collowing
contents:
@code 
test_data/AVG_test_data.bmp
test_data/markup1.bmp
img_000000000.fits
img_000000001.fits
img_000000002.fits
...
img_000000299.fits
@endcode

Then run the following command:

@code
./multispot5_headless --save_spots test_data/results.txt --log_ratios test_data/markup1.bmp test_data/img_000000*
@endcode

The program will save the results in the file \c test_data/results.txt . The
program will run indefinitely in the default setup, but you may view the
results at any stage.  There is no well defined stopping point for this type of
algorithm, so it is advisable continuously monitor the resultant image, and
stop the algorithm when the output image is no longer changing with time. After
30 minutes on a fast PC (e.g. Core i7 975), the ring structure which is not
resolved in the widefield image should be clearly visible.
After about 75 mins, the finer details of the structure begin to approach those seen in Fig 2e
in the associated paper.

You will have to process the results file further in order to view the results.
Run the following command:
@code
awk '/PASS/{for(i=2; i <=NF; i+=4)print $(i+2), $(i+3)}' test_data/results.txt > test_data/coordinates.txt
@endcode

The file <tt>test_data/coordinates.txt</tt> contains a long list of \f$(x, y)\f$
coordinates, representing possible spots positions. In order to view the
results, load the data into a graph plotting program and create a scatter plot.
NOTE: the axes are in pixel coordinates, so you will have to multiple any
distances by the number of nm/pixel in order to get distances in nm.


\section sExplaneExample Example usage explained


\subsection ssTestdata Test Data

The 300 TIFF files in the test directory correspond to the data used for Fig. 2
in the paper. Please refer to the paper for details on how the data was
obtained.

The file <tt>AVG_test_data.bmp</tt> is a Z projection made using 
<a href="http://rsbweb.nih.gov/ij/">ImageJ</a>.

The file <tt>markup1.bmp</tt> is a mask indicating which area of the image to
analyse. All perfectly black pixels are ignored, ecerything else is analysed. If
you overlay <tt>markup1.bmp</tt> and <tt>AVG_test_data.bmp</tt> you can see
which area the markup corresponds to. The markup file was created using 
<a hres="www.gimp.org">the GIMP</a>.

\subsection ssRunning Running the program

The general form for running the program is:

@code
./multispot5_headless [ --variable1 value1 [ --variable2 value 2 [ ... ] ] ] image1 image2 ... 
@endcode

so the example sets ths variable \c save_spots to \c test_data/results.txt and
the variable \c log_ratios to \c test_data/markup1.bmp.  The remaining
arguments is the list of files to be analysed.

The program gets the markup in the filename given in the \c log_ratios variable
(yes, the choice of name is very strange, and corresponds to a very old phase of
development). The more sanely named variable \c save_spots is the filename in
which the output is to be saved.

The program actually has a large number of variables which must be set. Most of
them you probably don't want to change, but some of them you will want to
change. The default values for these variables are stored in \c multispot5.cfg
The format of this file should be mostly self explanatory. Everything after a \c
// is a comment and is ignored.

Yu will probably want to change:
<ul>
<li> \c blur.mu

This is the prior over spot size, which is how the pixel size and microscope
FWHM are represented. Some example values for a FWHM of 300nm/pix at 160 and
100 nm/pix and for a FWHM of 270nm at 79nm per pixel. 

If you have significantly largre or smaller pixels, the performance may be
degraded.

<li> \c placement.uniform.num_spots

This is the initial number of spots to be placed down. Eventually, the algorithm
will converge to a reasonable number of spots, even if this value is far off.
The default value (15) is appropriate given the small area and dimness of the
sample data. You will want to increase this number for larger areas of markup
and relatively brighter regions.

If this number is more than 1000, then the algorithm will run very slowly and
may take several days.

</ul>

Note that variables specified on the commandline override all variables in the
configuration file.

The program can read FITS, BMP, PPM and PGM images. Depending on how it
has been compiled, it can also read TIFF, PNG and JPEG images.
The program cannot work on multi-image TIFF files. ImageJ can be used to split a
multi-image TIFF into a collection of single image files. All the images loaded
must be the same size.

\subsection ssExtract Extracting the data

The output file (in this case  \c test_data/results.txt ) containing the results
is in a format unsuitable for plotting directly, and must be extracted. The
reason for this is that the output file contains enough information to
seamlessly continue long runs which have been interrupted. The provided AWK
program extracts the coordinates of the spots over all iterations and puts them
in \c coordinated.txt.



*/
